#include "myevent.h"
#include <iostream>

constexpr int area_side(200);

MyArea::MyArea(): empty(false)
{
	set_content_width(area_side);
	set_content_height(area_side);
	
	set_draw_func(sigc::mem_fun(*this, &MyArea::on_draw));
}

MyArea::~MyArea()
{
}

void MyArea::clear()
{
	empty = true; 
	queue_draw();
}

void MyArea::draw()
{
	empty = false;
	queue_draw();
}

void MyArea::on_draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height)
{
	if(not empty)
	{
		// coordinates for the center of the window
		int xc, yc;
		xc = width / 2;
		yc = height / 2;

		cr->set_line_width(10.0);

		// draw red lines out from the center of the window
		cr->set_source_rgb(0.8, 0.0, 0.0);
		cr->move_to(0, 0);
		cr->line_to(xc, yc);
		cr->line_to(0, height);
		cr->move_to(xc, yc);
		cr->line_to(width, yc);
		cr->stroke();
	}
	else
	{
		std::cout << "Empty !" << std::endl;
	}
}

MyEvent::MyEvent():
	m_Main_Box(Gtk::Orientation::VERTICAL, 0),
	m_Buttons_Box(Gtk::Orientation::HORIZONTAL, 2),
	m_Button_Clear("Clear"),
	m_Button_Draw("Draw")
{
	set_title("Keyboard Events");
	set_resizable(false);
	set_child(m_Main_Box);

	m_Main_Box.append(m_Area);
	m_Main_Box.append(m_Buttons_Box);
	
	m_Buttons_Box.append(m_Button_Clear);
	m_Buttons_Box.append(m_Button_Draw);
	
	m_Button_Clear.signal_clicked().connect(
		sigc::mem_fun(*this, &MyEvent::on_button_clicked_clear));
		
    m_Button_Draw.signal_clicked().connect(
		sigc::mem_fun(*this, &MyEvent::on_button_clicked_draw));

    // Handling Keyboard Events.
    auto controller = Gtk::EventControllerKey::create();
    controller->signal_key_pressed().connect(
                  sigc::mem_fun(*this, &MyEvent::on_window_key_pressed), false);
    add_controller(controller);
}

bool MyEvent::on_window_key_pressed(guint keyval, guint, Gdk::ModifierType state)
{
	switch(gdk_keyval_to_unicode(keyval))
	{
		case 'w':
			std::cout << " Waow! key 'w' has been pressed !" << std::endl;
			return true;
		case 'c':
			std::cout << " action sur le lable du bouton Clear" 
				      << std::endl;
			m_Button_Clear.set_label("Clear");
			return true;
		case 'C':
			std::cout << " action sur le lable du bouton Clear" 
				      << std::endl;
			m_Button_Clear.set_label("CLEAR");
			return true;
		case 'q':
			std::cout << "Quit" << std::endl;
			hide(); //ou  exit(0);
			return true;
	}
    //the event has not been handled
    return false;
}

void MyEvent::on_button_clicked_clear()
{
	std::cout << "Clear" << std::endl;
	m_Area.clear();
}

void MyEvent::on_button_clicked_draw()
{
	std::cout << "Draw" << std::endl;
	m_Area.draw();
}
